use held_core::view::{colors::Colors, style::CharStyle};

use super::ModeRenderer;
use crate::{
    errors::*,
    view::status_data::{buffer_status_data, StatusLineData},
};
pub(super) struct DeleteRenderer;

impl ModeRenderer for DeleteRenderer {
    fn render(
        workspace: &mut crate::workspace::Workspace,
        monitor: &mut crate::view::monitor::Monitor,
        _mode: &mut super::ModeData,
    ) -> Result<()> {
        let mut presenter = monitor.build_presenter()?;

        if let Some(buffer) = &workspace.current_buffer {
            warn!("Delete buffer id: {}", buffer.id.unwrap());
            let data = buffer.data();
            presenter.print_buffer(buffer, &data, &workspace.syntax_set, None, None)?;

            let mode_name_data = StatusLineData {
                content: " DELETE ".to_string(),
                color: Colors::Inverted,
                style: CharStyle::Bold,
            };
            presenter.print_status_line(&[
                mode_name_data,
                buffer_status_data(&workspace.current_buffer),
            ])?;

            presenter.present()?;
        } else {
        }

        Ok(())
    }
}
